==> mule-metadata-model-persistence-1.1.5-sources.jar: org/mule/metadata/persistence/TypeAnnotationSerializer.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.metadata.persistence;

import static java.util.Optional.of;
import org.mule.metadata.api.annotation.AccessibilityAnnotation;
import org.mule.metadata.api.annotation.DefaultValueAnnotation;
import org.mule.metadata.api.annotation.EnumAnnotation;
import org.mule.metadata.api.annotation.ExampleAnnotation;
import org.mule.metadata.api.annotation.LabelAnnotation;
import org.mule.metadata.api.annotation.RegexPatternAnnotation;
import org.mule.metadata.api.annotation.TypeAnnotation;
import org.mule.metadata.api.annotation.TypeIdAnnotation;
import org.mule.metadata.persistence.api.TypeAnnotationSerializerExtenderFallback;
import org.mule.metadata.persistence.type.adapter.EnumTypeAnnotationTypeAdapter;
import org.mule.metadata.persistence.type.adapter.OnlyOneFieldTypeAdapterFactory;
import org.mule.metadata.persistence.type.adapter.OptionalTypeAdapterFactory;

import com.google.common.collect.HashBiMap;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.TypeAdapterFactory;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonWriter;

import java.util.Collection;
import java.util.Map;
import java.util.Optional;


/**
 * Serializer for {@link TypeAnnotation}.
 *
 * @since 1.0
 */
public final class TypeAnnotationSerializer {

  private static final String ERROR_DESERIALIZING_TYPE_ANNOTATION = "Error deserializing TypeAnnotation '%s'.";

  private final HashBiMap<String, Class<? extends TypeAnnotation>> stringClassHashMap = HashBiMap.create();
  private final Gson gson;
  private final TypeAnnotationSerializerExtenderFallback typeAnnotationSerializerExtenderFallback;

  public TypeAnnotationSerializer(Map<String, Class<? extends TypeAnnotation>> stingClassMapping,
                                  Collection<Object> additionalFeatures,
                                  TypeAnnotationSerializerExtenderFallback typeAnnotationSerializerExtenderFallback) {
    stringClassHashMap.putAll(stingClassMapping);
    GsonBuilder builder = new GsonBuilder()
        .registerTypeAdapterFactory(new OptionalTypeAdapterFactory())
        .registerTypeAdapter(EnumAnnotation.class, new EnumTypeAnnotationTypeAdapter())
        .registerTypeAdapterFactory(new OnlyOneFieldTypeAdapterFactory(AccessibilityAnnotation.class,
                                                                       LabelAnnotation.class,
                                                                       DefaultValueAnnotation.class,
                                                                       TypeIdAnnotation.class,
                                                                       RegexPatternAnnotation.class,
                                                                       ExampleAnnotation.class,
                                                                       DefaultValueAnnotation.class,
                                                                       RegexPatternAnnotation.class));

    additionalFeatures.forEach(feature -> {
      if (feature instanceof TypeAdapterFactory) {
        builder.registerTypeAdapterFactory((TypeAdapterFactory) feature);
      }
    });
    gson = builder.create();

    if (typeAnnotationSerializerExtenderFallback == null) {
      this.typeAnnotationSerializerExtenderFallback = annotationName -> {
        try {
          return of((Class<? extends TypeAnnotation>) Class.forName(annotationName));
        } catch (Exception e) {
          throw new RuntimeException(String.format(ERROR_DESERIALIZING_TYPE_ANNOTATION, annotationName), e);
        }
      };
    } else {
      this.typeAnnotationSerializerExtenderFallback = typeAnnotationSerializerExtenderFallback;
    }
  }

  /**
   * Deserializes a {@link TypeAnnotation} implementation.
   *
   * @param typeAnnotationName name of the {@link TypeAnnotation} that match with {@link TypeAnnotation#getName()} or
   *                           their full qualifier class name
   * @param json               {@link String} representation of the serialized {@link TypeAnnotation}
   * @return An {@link Optional<TypeAnnotation>} with the deserialized {@link TypeAnnotation} in case that the deserialization
   * could not be done, {@link Optional#empty()} will be returned
   */
  public <T extends TypeAnnotation> Optional<T> deserialize(String typeAnnotationName, String json) {
    Optional<T> optionalSerialization = Optional.empty();
    try {
      final Optional<Class<? extends TypeAnnotation>> annotationClass = getAnnotationClass(typeAnnotationName);
      if (annotationClass.isPresent()) {
        optionalSerialization = of((T) gson.getAdapter(annotationClass.get()).fromJson(json));
      }
    } catch (Exception e) {
      throw new RuntimeException(String.format(ERROR_DESERIALIZING_TYPE_ANNOTATION, typeAnnotationName), e);
    }

    return optionalSerialization;
  }

  /**
   * Serializes a {@link TypeAnnotation} implementation.
   *
   * @param jsonWriter     {@link JsonWriter} to write the serialized {@link TypeAnnotation}
   * @param typeAnnotation {@link TypeAnnotation} implementation to serialized
   */
  <T extends TypeAnnotation> void serialize(JsonWriter jsonWriter, T typeAnnotation) {
    gson.toJson(typeAnnotation, new TypeToken<T>() {}.getType(), jsonWriter);
  }

  private Optional<Class<? extends TypeAnnotation>> getAnnotationClass(String annotationName) {
    final Map<String, Class<? extends TypeAnnotation>> nameClassMapping = getNameClassMapping();
    Optional<Class<? extends TypeAnnotation>> optionalClass;

    if (nameClassMapping.containsKey(annotationName)) {
      optionalClass = of(nameClassMapping.get(annotationName));
    } else {
      optionalClass = typeAnnotationSerializerExtenderFallback.fallbackFor(annotationName);
    }
    return optionalClass;
  }

  /**
   * @return A {@link Map} that links a friendly name with a {@link TypeAnnotation} class.
   * This is helpful when deserializing a {@link TypeAnnotation} to be able to create the correct instance type.
   */
  Map<String, Class<? extends TypeAnnotation>> getNameClassMapping() {
    return stringClassHashMap;
  }

  /**
   * @return A {@link Map} that links a {@link TypeAnnotation} class with their friendly name.
   * This is helpful when serializing a {@link TypeAnnotation} to be able to tag it with a more friendly name that
   * the full qualifier name of the class.
   */
  Map<Class<? extends TypeAnnotation>, String> getClassNameMapping() {
    return stringClassHashMap.inverse();
  }
}

==> mule-metadata-model-persistence-1.3.0-20210622-sources.jar: org/mule/metadata/persistence/TypeAnnotationSerializer.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.metadata.persistence;

import static java.util.Optional.of;

import org.mule.metadata.api.annotation.AccessibilityAnnotation;
import org.mule.metadata.api.annotation.DefaultValueAnnotation;
import org.mule.metadata.api.annotation.EnumAnnotation;
import org.mule.metadata.api.annotation.ExampleAnnotation;
import org.mule.metadata.api.annotation.LabelAnnotation;
import org.mule.metadata.api.annotation.RegexPatternAnnotation;
import org.mule.metadata.api.annotation.TypeAnnotation;
import org.mule.metadata.api.annotation.TypeIdAnnotation;
import org.mule.metadata.persistence.api.TypeAnnotationSerializerExtenderFallback;
import org.mule.metadata.persistence.type.adapter.EnumTypeAnnotationTypeAdapter;
import org.mule.metadata.persistence.type.adapter.OnlyOneFieldTypeAdapterFactory;
import org.mule.metadata.persistence.type.adapter.OptionalTypeAdapterFactory;

import java.util.Collection;
import java.util.Map;
import java.util.Optional;

import com.google.common.collect.HashBiMap;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.TypeAdapterFactory;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonWriter;


/**
 * Serializer for {@link TypeAnnotation}.
 *
 * @since 1.0
 */
public final class TypeAnnotationSerializer {

  private static final String ERROR_DESERIALIZING_TYPE_ANNOTATION = "Error deserializing TypeAnnotation '%s'.";

  private final HashBiMap<String, Class<? extends TypeAnnotation>> stringClassHashMap = HashBiMap.create();
  private final Gson gson;
  private final TypeAnnotationSerializerExtenderFallback typeAnnotationSerializerExtenderFallback;

  public TypeAnnotationSerializer(Map<String, Class<? extends TypeAnnotation>> stingClassMapping,
                                  Collection<Object> additionalFeatures,
                                  TypeAnnotationSerializerExtenderFallback typeAnnotationSerializerExtenderFallback) {
    stringClassHashMap.putAll(stingClassMapping);
    GsonBuilder builder = new GsonBuilder()
        .registerTypeAdapterFactory(new OptionalTypeAdapterFactory())
        .registerTypeAdapter(EnumAnnotation.class, new EnumTypeAnnotationTypeAdapter())
        .registerTypeAdapterFactory(new OnlyOneFieldTypeAdapterFactory(AccessibilityAnnotation.class,
                                                                       LabelAnnotation.class,
                                                                       DefaultValueAnnotation.class,
                                                                       TypeIdAnnotation.class,
                                                                       RegexPatternAnnotation.class,
                                                                       ExampleAnnotation.class,
                                                                       DefaultValueAnnotation.class,
                                                                       RegexPatternAnnotation.class));

    additionalFeatures.forEach(feature -> {
      if (feature instanceof TypeAdapterFactory) {
        builder.registerTypeAdapterFactory((TypeAdapterFactory) feature);
      }
    });
    gson = builder.create();

    if (typeAnnotationSerializerExtenderFallback == null) {
      this.typeAnnotationSerializerExtenderFallback = annotationName -> {
        try {
          return of((Class<? extends TypeAnnotation>) Class.forName(annotationName));
        } catch (Exception e) {
          throw new RuntimeException(String.format(ERROR_DESERIALIZING_TYPE_ANNOTATION, annotationName), e);
        }
      };
    } else {
      this.typeAnnotationSerializerExtenderFallback = typeAnnotationSerializerExtenderFallback;
    }
  }

  /**
   * Deserializes a {@link TypeAnnotation} implementation.
   *
   * @param typeAnnotationName name of the {@link TypeAnnotation} that match with {@link TypeAnnotation#getName()} or
   *                           their full qualifier class name
   * @param json               {@link String} representation of the serialized {@link TypeAnnotation}
   * @return An {@link Optional<TypeAnnotation>} with the deserialized {@link TypeAnnotation} in case that the deserialization
   * could not be done, {@link Optional#empty()} will be returned
   */
  public <T extends TypeAnnotation> Optional<T> deserialize(String typeAnnotationName, String json) {
    Optional<T> optionalSerialization = Optional.empty();
    try {
      final Optional<Class<? extends TypeAnnotation>> annotationClass = getAnnotationClass(typeAnnotationName);
      if (annotationClass.isPresent()) {
        optionalSerialization = of((T) gson.getAdapter(annotationClass.get()).fromJson(json));
      }
    } catch (Exception e) {
      throw new RuntimeException(String.format(ERROR_DESERIALIZING_TYPE_ANNOTATION, typeAnnotationName), e);
    }

    return optionalSerialization;
  }

  /**
   * Serializes a {@link TypeAnnotation} implementation.
   *
   * @param jsonWriter     {@link JsonWriter} to write the serialized {@link TypeAnnotation}
   * @param typeAnnotation {@link TypeAnnotation} implementation to serialized
   */
  public <T extends TypeAnnotation> void serialize(JsonWriter jsonWriter, T typeAnnotation) {
    gson.toJson(typeAnnotation, new TypeToken<T>() {}.getType(), jsonWriter);
  }

  private Optional<Class<? extends TypeAnnotation>> getAnnotationClass(String annotationName) {
    final Map<String, Class<? extends TypeAnnotation>> nameClassMapping = getNameClassMapping();
    Optional<Class<? extends TypeAnnotation>> optionalClass;

    if (nameClassMapping.containsKey(annotationName)) {
      optionalClass = of(nameClassMapping.get(annotationName));
    } else {
      optionalClass = typeAnnotationSerializerExtenderFallback.fallbackFor(annotationName);
    }
    return optionalClass;
  }

  /**
   * @return A {@link Map} that links a friendly name with a {@link TypeAnnotation} class.
   * This is helpful when deserializing a {@link TypeAnnotation} to be able to create the correct instance type.
   */
  public Map<String, Class<? extends TypeAnnotation>> getNameClassMapping() {
    return stringClassHashMap;
  }

  /**
   * @return A {@link Map} that links a {@link TypeAnnotation} class with their friendly name.
   * This is helpful when serializing a {@link TypeAnnotation} to be able to tag it with a more friendly name that
   * the full qualifier name of the class.
   */
  public Map<Class<? extends TypeAnnotation>, String> getClassNameMapping() {
    return stringClassHashMap.inverse();
  }
}
